Package: taxize
Title: Taxonomic Information from Around the Web
Description: Interacts with a suite of web application programming interfaces
    (API) for taxonomic tasks, such as getting database specific taxonomic
    identifiers, verifying species names, getting taxonomic hierarchies, fetching
    downstream and upstream taxonomic names, getting taxonomic synonyms, converting
    scientific to common names and vice versa, and more. Some of the services
    supported include 'NCBI E-utilities' (<https://www.ncbi.nlm.nih.gov/books/NBK25501/>),
    'Encyclopedia of Life' (<https://eol.org/docs/what-is-eol/data-services>),
    'Global Biodiversity Information Facility' (<https://techdocs.gbif.org/en/openapi/>),
    and many more. Links to the API documentation for other supported services are
    available in the documentation for their respective functions in this package.
Version: 0.10.1
License: MIT + file LICENSE
URL: https://docs.ropensci.org/taxize/ (website),
        https://github.com/ropensci/taxize (devel)
BugReports: https://github.com/ropensci/taxize/issues
Authors@R: 
    c(person(given = "Scott",
             family = "Chamberlain",
             role = c("aut"),
             email = "myrmecocystus@gmail.com"),
      person(given = "Eduard",
             family = "Szoecs",
             role = "aut"),
      person(given = "Zachary",
             family = "Foster",
             role = c("aut", "cre"),
             email = "zacharyfoster1989@gmail.com"),
      person(given = "Zebulun",
             family = "Arendsee",
             role = "aut"),
      person(given = "Carl",
             family = "Boettiger",
             role = "ctb"),
      person(given = "Karthik",
             family = "Ram",
             role = "ctb"),
      person(given = "Ignasi",
             family = "Bartomeus",
             role = "ctb"),
      person(given = "John",
             family = "Baumgartner",
             role = "ctb"),
      person(given = "James",
             family = "O'Donnell",
             role = "ctb"),
      person(given = "Jari",
             family = "Oksanen",
             role = "ctb"),
      person(given = "Bastian Greshake",
             family = "Tzovaras",
             role = "ctb"),
      person(given = "Philippe",
             family = "Marchand",
             role = "ctb"),
      person(given = "Vinh",
             family = "Tran",
             role = "ctb"),
      person(given = "Maëlle",
             family = "Salmon",
             role = "ctb"),
      person(given = "Gaopeng",
             family = "Li",
             role = "ctb"),
      person(given = "Matthias",
             family = "Grenié",
             role = "ctb"),
      person(given = "rOpenSci",
             role = "fnd"))
LazyLoad: yes
LazyData: true
Encoding: UTF-8
Language: en-US
Depends: R(>= 3.2.1)
Imports: graphics, methods, stats, utils, crul (>= 0.7.0), xml2 (>=
        1.2.0), jsonlite, ape, zoo, data.table, tibble (>= 1.2),
        rredlist, rotl (>= 3.0.0), ritis (>= 0.7.6), worrms (>= 0.4.0),
        natserv (>= 1.0.0), wikitaxa (>= 0.3.0), R6, crayon, cli,
        phangorn, lifecycle, curl, stringi
Suggests: testthat, vegan, vcr
RoxygenNote: 7.3.3
X-schema.org-applicationCategory: Taxonomy
X-schema.org-keywords: taxonomy, biology, nomenclature, JSON, API, web,
        api-client, identifiers, species, names
X-schema.org-isPartOf: https://ropensci.org
NeedsCompilation: no
Packaged: 2026-02-13 20:17:14 UTC; fosterz
Author: Scott Chamberlain [aut],
  Eduard Szoecs [aut],
  Zachary Foster [aut, cre],
  Zebulun Arendsee [aut],
  Carl Boettiger [ctb],
  Karthik Ram [ctb],
  Ignasi Bartomeus [ctb],
  John Baumgartner [ctb],
  James O'Donnell [ctb],
  Jari Oksanen [ctb],
  Bastian Greshake Tzovaras [ctb],
  Philippe Marchand [ctb],
  Vinh Tran [ctb],
  Maëlle Salmon [ctb],
  Gaopeng Li [ctb],
  Matthias Grenié [ctb],
  rOpenSci [fnd]
Maintainer: Zachary Foster <zacharyfoster1989@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-14 14:40:02 UTC
Built: R 4.6.0; ; 2026-04-29 21:35:31 UTC; unix
