# Generated by roxygen2: do not edit by hand

S3method(as.boldid,boldid)
S3method(as.boldid,character)
S3method(as.boldid,data.frame)
S3method(as.boldid,list)
S3method(as.boldid,numeric)
S3method(as.data.frame,boldid)
S3method(as.data.frame,eolid)
S3method(as.data.frame,gbifid)
S3method(as.data.frame,iucn)
S3method(as.data.frame,natservid)
S3method(as.data.frame,nbnid)
S3method(as.data.frame,pow)
S3method(as.data.frame,tolid)
S3method(as.data.frame,tpsid)
S3method(as.data.frame,tsn)
S3method(as.data.frame,uid)
S3method(as.data.frame,wiki)
S3method(as.data.frame,wormsid)
S3method(as.eolid,character)
S3method(as.eolid,data.frame)
S3method(as.eolid,eolid)
S3method(as.eolid,list)
S3method(as.eolid,numeric)
S3method(as.gbifid,character)
S3method(as.gbifid,data.frame)
S3method(as.gbifid,gbifid)
S3method(as.gbifid,list)
S3method(as.gbifid,numeric)
S3method(as.iucn,character)
S3method(as.iucn,data.frame)
S3method(as.iucn,iucn)
S3method(as.iucn,list)
S3method(as.iucn,numeric)
S3method(as.natservid,character)
S3method(as.natservid,data.frame)
S3method(as.natservid,list)
S3method(as.natservid,natservid)
S3method(as.natservid,numeric)
S3method(as.nbnid,character)
S3method(as.nbnid,data.frame)
S3method(as.nbnid,list)
S3method(as.nbnid,nbnid)
S3method(as.pow,character)
S3method(as.pow,data.frame)
S3method(as.pow,list)
S3method(as.pow,pow)
S3method(as.tolid,character)
S3method(as.tolid,data.frame)
S3method(as.tolid,list)
S3method(as.tolid,numeric)
S3method(as.tolid,tolid)
S3method(as.tpsid,character)
S3method(as.tpsid,data.frame)
S3method(as.tpsid,list)
S3method(as.tpsid,numeric)
S3method(as.tpsid,tpsid)
S3method(as.tsn,character)
S3method(as.tsn,data.frame)
S3method(as.tsn,list)
S3method(as.tsn,numeric)
S3method(as.tsn,tsn)
S3method(as.uid,character)
S3method(as.uid,data.frame)
S3method(as.uid,list)
S3method(as.uid,numeric)
S3method(as.uid,uid)
S3method(as.wiki,character)
S3method(as.wiki,data.frame)
S3method(as.wiki,list)
S3method(as.wiki,numeric)
S3method(as.wiki,wiki)
S3method(as.wormsid,character)
S3method(as.wormsid,data.frame)
S3method(as.wormsid,list)
S3method(as.wormsid,numeric)
S3method(as.wormsid,wormsid)
S3method(cbind,classification)
S3method(cbind,classification_ids)
S3method(children,boldid)
S3method(children,default)
S3method(children,ids)
S3method(children,tsn)
S3method(children,uid)
S3method(children,wormsid)
S3method(classification,boldid)
S3method(classification,default)
S3method(classification,eolid)
S3method(classification,gbifid)
S3method(classification,ids)
S3method(classification,natservid)
S3method(classification,nbnid)
S3method(classification,pow)
S3method(classification,tolid)
S3method(classification,tpsid)
S3method(classification,tsn)
S3method(classification,uid)
S3method(classification,wiki)
S3method(classification,wormsid)
S3method(comm2sci,default)
S3method(comm2sci,tsn)
S3method(comm2sci,uid)
S3method(downstream,boldid)
S3method(downstream,default)
S3method(downstream,gbifid)
S3method(downstream,ids)
S3method(downstream,tsn)
S3method(downstream,uid)
S3method(downstream,wormsid)
S3method(id2name,boldid)
S3method(id2name,default)
S3method(id2name,gbifid)
S3method(id2name,tolid)
S3method(id2name,tsn)
S3method(id2name,uid)
S3method(id2name,wormsid)
S3method(iucn_status,default)
S3method(iucn_status,iucn_summary)
S3method(iucn_summary,character)
S3method(iucn_summary,default)
S3method(lowest_common,default)
S3method(lowest_common,gbifid)
S3method(lowest_common,tolid)
S3method(lowest_common,tsn)
S3method(lowest_common,uid)
S3method(plot,classtree)
S3method(print,classtree)
S3method(print,tax_agg)
S3method(rbind,classification)
S3method(rbind,classification_ids)
S3method(sci2comm,default)
S3method(sci2comm,iucn)
S3method(sci2comm,tsn)
S3method(sci2comm,uid)
S3method(sci2comm,wormsid)
S3method(synonyms,default)
S3method(synonyms,ids)
S3method(synonyms,iucn)
S3method(synonyms,nbnid)
S3method(synonyms,pow)
S3method(synonyms,tpsid)
S3method(synonyms,tsn)
S3method(synonyms,wormsid)
S3method(synonyms_df,default)
S3method(synonyms_df,synonyms)
S3method(tax_rank,character)
S3method(tax_rank,default)
S3method(tax_rank,numeric)
S3method(upstream,default)
S3method(upstream,ids)
S3method(upstream,tsn)
export(apgFamilies)
export(apgOrders)
export(apg_lookup)
export(as.boldid)
export(as.colid)
export(as.eolid)
export(as.gbifid)
export(as.iucn)
export(as.natservid)
export(as.nbnid)
export(as.pow)
export(as.tolid)
export(as.tpsid)
export(as.tsn)
export(as.ubioid)
export(as.uid)
export(as.wiki)
export(as.wormsid)
export(bold_children)
export(bold_downstream)
export(bold_ping)
export(bold_search)
export(children)
export(class2tree)
export(classification)
export(col_children)
export(col_classification)
export(col_downstream)
export(col_ping)
export(col_search)
export(comm2sci)
export(downstream)
export(eol_dataobjects)
export(eol_hierarchy)
export(eol_invasive)
export(eol_pages)
export(eol_ping)
export(eol_search)
export(eubon)
export(eubon_capabilities)
export(eubon_children)
export(eubon_hierarchy)
export(eubon_search)
export(fg_all_updated_names)
export(fg_author_search)
export(fg_deprecated_names)
export(fg_epithet_search)
export(fg_name_by_key)
export(fg_name_full_by_lsid)
export(fg_name_search)
export(fg_ping)
export(gbif_downstream)
export(gbif_name_usage)
export(gbif_parse)
export(gbif_ping)
export(genbank2uid)
export(get_boldid)
export(get_boldid_)
export(get_colid)
export(get_colid_)
export(get_eolid)
export(get_eolid_)
export(get_gbifid)
export(get_gbifid_)
export(get_genes)
export(get_genes_avail)
export(get_ids)
export(get_ids_)
export(get_iucn)
export(get_natservid)
export(get_natservid_)
export(get_nbnid)
export(get_nbnid_)
export(get_pow)
export(get_pow_)
export(get_seqs)
export(get_tolid)
export(get_tolid_)
export(get_tpsid)
export(get_tpsid_)
export(get_tsn)
export(get_tsn_)
export(get_ubioid)
export(get_ubioid_)
export(get_uid)
export(get_uid_)
export(get_wiki)
export(get_wiki_)
export(get_wormsid)
export(get_wormsid_)
export(getkey)
export(gisd_isinvasive)
export(gna_data_sources)
export(gna_parse)
export(gna_search)
export(gna_verifier)
export(gni_details)
export(gni_parse)
export(gni_seach)
export(gnr_datasources)
export(gnr_resolve)
export(id2name)
export(ion)
export(iplant_resolve)
export(ipni_ping)
export(ipni_search)
export(itis_acceptname)
export(itis_downstream)
export(itis_getrecord)
export(itis_hierarchy)
export(itis_kingdomnames)
export(itis_lsid)
export(itis_name)
export(itis_native)
export(itis_ping)
export(itis_refs)
export(itis_taxrank)
export(itis_terms)
export(iucn_getname)
export(iucn_id)
export(iucn_status)
export(iucn_summary)
export(lowest_common)
export(names_list)
export(nbn_classification)
export(nbn_ping)
export(nbn_search)
export(nbn_synonyms)
export(ncbi_children)
export(ncbi_downstream)
export(ncbi_get_taxon_summary)
export(ncbi_getbyid)
export(ncbi_getbyname)
export(ncbi_ping)
export(ncbi_search)
export(phylomatic_format)
export(phylomatic_tree)
export(plantminer)
export(pow_lookup)
export(pow_search)
export(pow_synonyms)
export(rankagg)
export(resolve)
export(sci2comm)
export(scrapenames)
export(status_codes)
export(synonyms)
export(synonyms_df)
export(tax_agg)
export(tax_name)
export(tax_rank)
export(taxize_capwords)
export(taxize_cite)
export(taxize_ldfast)
export(taxize_options)
export(taxon_clear)
export(taxon_last)
export(tnrs)
export(tnrs_sources)
export(tol_resolve)
export(tp_acceptednames)
export(tp_accnames)
export(tp_classification)
export(tp_dist)
export(tp_namedistributions)
export(tp_namereferences)
export(tp_refs)
export(tp_search)
export(tp_summary)
export(tp_synonyms)
export(tpl_families)
export(tpl_get)
export(tpl_search)
export(tropicos_ping)
export(ubio_classification)
export(ubio_classification_search)
export(ubio_id)
export(ubio_ping)
export(ubio_search)
export(ubio_synonyms)
export(upstream)
export(use_entrez)
export(use_eol)
export(use_iucn)
export(use_tropicos)
export(vascan_ping)
export(vascan_search)
export(worms_downstream)
importFrom(R6,R6Class)
importFrom(ape,as.phylo.hclust)
importFrom(ape,plot.phylo)
importFrom(ape,read.tree)
importFrom(cli,cat_line)
importFrom(cli,rule)
importFrom(cli,symbol)
importFrom(crayon,style)
importFrom(crul,AsyncVaried)
importFrom(crul,HttpClient)
importFrom(crul,HttpRequest)
importFrom(crul,upload)
importFrom(data.table,as.data.table)
importFrom(data.table,dcast)
importFrom(data.table,melt)
importFrom(data.table,rbindlist)
importFrom(data.table,setDF)
importFrom(data.table,transpose)
importFrom(graphics,plot)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(methods,as)
importFrom(methods,is)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,hclust)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(utils,URLencode)
importFrom(utils,browseURL)
importFrom(utils,citation)
importFrom(utils,download.file)
importFrom(utils,read.delim)
importFrom(utils,tail)
importFrom(utils,write.table)
importFrom(xml2,as_list)
importFrom(xml2,read_xml)
importFrom(xml2,xml_children)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_name)
importFrom(xml2,xml_ns)
importFrom(xml2,xml_text)
importFrom(zoo,na.locf)
