
# taxize

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![rworkflows](https://github.com/ropensci/taxize/workflows/rworkflows/badge.svg)](https://github.com/ropensci/taxize/actions/)
[![codecov](https://codecov.io/gh/ropensci/taxize/branch/master/graph/badge.svg)](https://app.codecov.io/gh/ropensci/taxize)
[![rstudio mirror
downloads](https://cranlogs.r-pkg.org/badges/taxize)](https://github.com/r-hub/cranlogs.app)
[![cran
version](https://www.r-pkg.org/badges/version/taxize)](https://cran.r-project.org/package=taxize)

`taxize` allows users to search over many taxonomic data sources for
species names (scientific and common) and download up and downstream
taxonomic hierarchical information - among other things.

Package documentation: <https://docs.ropensci.org/taxize/>

## Installation

### Stable version from CRAN

``` r
install.packages("taxize")
```

### Development version from GitHub

Windows users install Rtools first.

``` r
install.packages("remotes")
remotes::install_github("ropensci/taxize")
```

``` r
library('taxize')
```

## Screencast

<a href="https://vimeo.com/92883063"><img src="man/figures/screencast.png" width="400"></a>

## Contributing

See our
[CONTRIBUTING](https://github.com/ropensci/taxize/blob/master/.github/CONTRIBUTING.md)
document.

## Contributors

Collected via GitHub Issues: honors all contributors in alphabetical
order. Code contributors are in bold.

afkoeppel - ahhurlbert - albnd - Alectoria - andzandz11 - **anirvan** -
antagomir - **arendsee** - ArielGreiner - arw36 - ashenkin -
**ashiklom** - benjaminschwetz - **benmarwick** - bienflorencia -
binkySallly - bomeara - BridgettCollis - bw4sz - **cboettig** -
cdeterman - ChrKoenig - chuckrp - clarson2191 - claudenozeres -
cmzambranat - cparsania - daattali - DanielGMead - DarrenObbard -
davharris - davidvilanova - diogoprov - **dlebauer** - dlenz1 -
dougwyu - dschlaep - **EDiLD** - edwbaker - emhart - eregenyi -
fdschneider - fgabriel1891 - fischhoff - **fmichonneau** - **fozy81** -
**gedankenstuecke** - gimoya - git-og - glaroc - **gpli** - gustavobio -
hlapp - **ibartomeus** - **Ironholds** - jabard89 - jangorecki -
**jarioksa** - jebyrnes - **jeroen** - **jimmyodonnell** - joelnitta -
johnbaums - jonmcalder - jordancasey - **josephwb** - jsgosnell -
JulietteLgls - **jwilk** - kamapu - **karthik** - **katrinleinweber** -
KevCaz - kgturner - kmeverson - Koalha - **ljvillanueva** - **maelle** -
Markus2015 - matutosi - mcsiple - MikkoVihtakari - millerjef -
miriamgrace - mpnelsen - MUSEZOOLVERT - nate-d-olson - nmatzke - npch -
ocstringham - p-neves - p-schaefer - padpadpadpad - paternogbc -
**patperu** - pederengelstad - philippi - Phylloxera - **pmarchand1** -
pozsgaig - pssguy - **raredd** - rec3141 - **Rekyt** - RodgerG -
rossmounce - sariya - sastoudt - scelmendorf - **sckott** -
SimonGoring - snsheth - snubian - Squiercg - sunray1 - **taddallas** -
tdjames1 - tmkurobe - toczydlowski - tpaulson1 - tpoisot -
**TrashBirdEcology** - **trvinh** - **vijaybarve** - wcornwell -
willpearse - wpetry - yhg926 - **zachary-foster**

## Road map

Check out our
[milestones](https://github.com/ropensci/taxize/milestones) to see what
we plan to get done for each version.

## Meta

- Please [report any issues or
  bugs](https://github.com/ropensci/taxize/issues).
- License: MIT
- Get citation information for `taxize` in R doing
  `citation(package = 'taxize')`
- Please note that this package is released with a [Contributor Code of
  Conduct](https://ropensci.org/code-of-conduct/). By contributing to
  this project, you agree to abide by its terms.

[![rofooter](https://ropensci.org/public_images/github_footer.png)](https://ropensci.org)
