% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eol_hierarchy.R
\name{eol_hierarchy}
\alias{eol_hierarchy}
\title{Retrieve the taxonomic hierarchy from given EOL taxonID.}
\usage{
eol_hierarchy(...)
}
\description{
THIS FUNCTION IS DEFUNCT.
}
\keyword{internal}
