% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gna_parse.R
\name{gni_parse}
\alias{gni_parse}
\title{Parse scientific names using EOL's name parser.}
\usage{
gni_parse(names, ...)
}
\description{
THIS FUNCTION IS DEFUNCT.
}
\keyword{internal}
