% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ncbi_getbyname.R
\name{get_seqs}
\alias{get_seqs}
\title{Retrieve gene sequences from NCBI by accession number.}
\usage{
get_seqs(...)
}
\description{
THIS FUNCTION IS DEFUNCT.
}
\keyword{internal}
