% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/key_helpers.R
\name{use_eol-defunct}
\alias{use_eol-defunct}
\alias{use_eol}
\title{use eol}
\usage{
use_eol()
}
\description{
use eol
}
\keyword{internal}
