% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gna_search.R
\name{gni_seach}
\alias{gni_seach}
\title{Search for taxonomic names using the Global Names Index}
\usage{
gni_seach(names, ...)
}
\description{
THIS FUNCTION IS DEFUNCT.
}
\keyword{internal}
