% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iucn_summary.R
\name{iucn_summary_id}
\alias{iucn_summary_id}
\title{DEFUNCT}
\usage{
iucn_summary_id(...)
}
\description{
DEFUNCT
}
\keyword{internal}
