% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ncbi_search.R
\name{ncbi_search}
\alias{ncbi_search}
\title{Search for gene sequences available for taxa from NCBI.}
\usage{
ncbi_search(...)
}
\description{
THIS FUNCTION IS DEFUNCT.
}
\keyword{internal}
