% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ubio_id.R
\name{ubio_id}
\alias{ubio_id}
\title{Search uBio by namebank ID.}
\usage{
ubio_id(...)
}
\arguments{
\item{...}{Parameters, ignored}
}
\description{
THIS FUNCTION IS DEFUNCT.
}
\keyword{internal}
