% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/status_codes.R
\name{status_codes}
\alias{status_codes}
\title{Get HTTP status codes}
\usage{
status_codes()
}
\description{
Get HTTP status codes
}
\examples{
status_codes()
}
\seealso{
\code{\link[=ping]{ping()}}
}
