% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ncbi_search.R
\name{get_genes_avail}
\alias{get_genes_avail}
\title{Retrieve gene sequences from NCBI by accession number.}
\usage{
get_genes_avail(...)
}
\description{
THIS FUNCTION IS DEFUNCT.
}
\keyword{internal}
