% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eubon_capabilities.R
\name{eubon_capabilities}
\alias{eubon_capabilities}
\title{EUBON capabilities}
\usage{
eubon_capabilities(...)
}
\arguments{
\item{...}{Curl options passed on to \link[crul:verb-GET]{crul::verb-GET}}
}
\description{
EUBON capabilities
}
\examples{
\dontrun{
eubon_capabilities()
}
}
\references{
https://cybertaxonomy.eu/eu-bon/utis/1.3/doc.html
}
\seealso{
Other eubon-methods: 
\code{\link{eubon_children}()},
\code{\link{eubon_hierarchy}()},
\code{\link{eubon_search}()}
}
\concept{eubon-methods}
