% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ncbi_getbyid.R
\name{get_genes}
\alias{get_genes}
\title{Retrieve gene sequences from NCBI by accession number.}
\usage{
get_genes(...)
}
\description{
THIS FUNCTION IS DEFUNCT.
}
\keyword{internal}
