% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tpl_search.r
\name{tpl_search}
\alias{tpl_search}
\title{A light wrapper around the taxonstand fxn to call Theplantlist.org database.}
\usage{
tpl_search()
}
\description{
THIS FUNCTION IS DEFUNCT.
}
